import warnings


class AlreadyRegistered(Exception):
    pass


class NotRegistered(KeyError):
    pass


class Registry(dict):
    """
    Dictionary like object representing a collection of objects.

    To add items to the registry, decorate the class with the ``register``
    method.

    my_registry  = Registry()

    @my_registry.register
    class Example(object):
        key = 'HDR0'


    Then when you'll be able to get the registered class back out of the
    registry by accessing it like a dictionary with the key
    >>> my_registry['HDR0']
    <class 'path.to.module.Example'>

    """

    def __init__(self, key_name='key', *args, **kwargs):
        self.key_name = key_name
        super(Registry, self).__init__(*args, **kwargs)

    def register(self, klass):
        key = self._get_key_from_class(klass)
        if key in self:
            msg = "Key '{key}' has already been registered as '{name}'.".format(key=key, name=self[key].__name__)
            raise AlreadyRegistered(msg)

        self.__setitem__(key, klass)
        return klass

    def unregister(self, klass):
        key = self._get_key_from_class(klass)
        if key in self:
            self.__delitem__(key)

    def _get_key_from_class(self, klass):
        return getattr(klass, self.key_name)

    def __getitem__(self, key):
        """
        The registry is a dictionary. Raising `NotRegistered` instead of
        a `KeyError` seems more appropriate.
        """
        if key in self:
            return super(Registry, self).__getitem__(key)

        raise NotRegistered("Key '{key}' has not been registered.".format(key=key))

    def get_registered_class(self, key):
        warnings.warn(
            "The 'get_registered_class' method will be removed soon. Just access your registry like a dictionary.",
            PendingDeprecationWarning,
        )
        return self[key]

    @property
    def classes(self):
        warnings.warn(
            "The 'classes' property will be removed soon. Just access your registry like a dictionary.",
            PendingDeprecationWarning,
        )
        return self



import os
from os import path


class AutoImport(object):

    def __init__(self, module):
        self.module = module

    def get_package_name(self, package_file_path):
        dotted_path = package_file_path.replace("/", ".")
        idx = dotted_path.index(self.module.__name__.replace("..", "."))
        return dotted_path[idx:]

    def get_module_name(self, f):
        return path.basename(f)[:-3]

    def filter_out_init_and_non_python_files(self, file_name):
        return file_name.endswith(".py") and not file_name.startswith("__")

    def setup(self):
        module_files = os.walk(path.abspath(path.dirname(self.module.__file__)))

        for file_path, _, files in module_files:
            for python_file_name in filter(self.filter_out_init_and_non_python_files, files):
                __import__(self.get_package_name(file_path) + "." + self.get_module_name(python_file_name))



from unittest import TestCase

from class_registry.auto_import import AutoImport
from class_registry import Registry, AlreadyRegistered, NotRegistered


class FakeModule(object):

    def __init__(self, name):
        self.name = name

    @property
    def __name__(self):
        return self.name


class RegistryTests(TestCase):

    def get_test_class(self, key="ABC"):
        return type('TestClass', (object, ), {'key': key})

    def test_register_adds_item_to_collection(self):
        lc = Registry()
        TestClass = self.get_test_class("ABC")
        lc.register(TestClass)
        self.assertEqual(lc["ABC"], TestClass)

    def test_unregister_removes_item_from_collection(self):
        key = "ABC"
        lc = Registry()
        TestClass = self.get_test_class(key)
        lc[key] = TestClass
        lc.unregister(TestClass)
        self.assertNotIn(key, lc)

    def test_unregister_returns_none_when_class_isnt_already_registered(self):
        # as opposed to blowing up on a KeyError
        lc = Registry()
        TestClass = self.get_test_class("ABC")

        self.assertEqual(None, lc.unregister(TestClass))

    def test_raises_already_registered_when_key_has_already_been_registered(self):
        test_class_one = self.get_test_class("ABC")
        test_class_two = type('TestClassTwo', (object, ), {'key': "ABC"})
        lc = Registry()
        lc.register(test_class_one)

        with self.assertRaises(AlreadyRegistered) as e:
            lc.register(test_class_two)
        self.assertEqual("Key 'ABC' has already been registered as 'TestClass'.", str(e.exception))

    def test_raises_not_registered_when_trying_to_access_an_item_that_has_not_been_registered(self):
        lc = Registry()
        with self.assertRaises(NotRegistered) as e:
            lc['ABC']
        self.assertEqual('"Key \'ABC\' has not been registered."', str(e.exception))

    def test_classes_property_returns_same_object(self):
        # for backwards compatibility. Now that registry is a dict, the old
        # classes property has no purpose.
        lc = Registry()
        classes = lc.classes
        self.assertEqual(lc, classes)

    def test_get_registered_class_returns_registered_class(self):
        # for backwards compatibility. You can (and should) just access
        # your registry like a normal dictionary.
        lc = Registry()
        TestClass = self.get_test_class("ABC")
        lc.register(TestClass)
        self.assertEqual(lc.get_registered_class("ABC"), TestClass)

    def test_registry_allows_customizable_key_name(self):
        lc = Registry(key_name='obj_code')
        TestClass = type('TestClass', (object, ), {'obj_code': "code1"})
        lc.register(TestClass)
        self.assertEqual(lc["code1"], TestClass)


class AutoImportTests(TestCase):

    def test_will_replace_any_slash_in_the_path_with_a_dot_instead(self):
        fake_module = FakeModule(name='project')
        auto_import = AutoImport(fake_module)
        package_name = auto_import.get_package_name('foo/project')
        self.assertEqual(package_name, 'project')

    def test_will_replace_two_dots_in_the_module_name_when_running_south_migrations(self):
        fake_module = FakeModule(name='project..app')
        auto_import = AutoImport(fake_module)
        package_name = auto_import.get_package_name('foo/project/app')
        self.assertEqual(package_name, 'project.app')

    def test_when_demo_is_imported_registry_contains_all_classes_under_action_package(self):
        from class_registry.tests.demo.actions import action_registry as actions
        self.assertIn('one', actions)
        self.assertIn('two', actions)
        self.assertIn('three', actions)



from setuptools import setup, find_packages

setup(
    name="django-class-registry",
    version='1.1.2',
    author="IMT Computer Services",
    author_email="imtapps@apps-system.com",
    description="Register any classes like Django's admin.",
    long_description=open('README.txt', 'r').read(),
    url="http://github.com/imtapps/django-class-registry",
    packages=find_packages(),
    install_requires=[],
    tests_requires=[],
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Topic :: Software Development",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
    ],
)
